package com.catarc.traceability.repository;

import java.util.List;
import java.util.Map;

import org.apache.ibatis.annotations.Insert;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.SelectKey;
import org.springframework.stereotype.Repository;
import org.apache.ibatis.annotations.Update;

import com.catarc.traceability.entity.Role;
import com.catarc.traceability.entity.User;
import com.catarc.traceability.repository.base.BaseRepository;
import com.isoft91.web.base.repository.DynamicQueryRepository;

@Repository
public interface UserRepository extends BaseRepository<User>,DynamicQueryRepository{
	
	/**
	 * 登录
	 * @MethodName:login
	 * @author: wangyao
	 * @email: 842261834@qq.com 
	 * @date 2017年8月29日 下午1:33:56
	 * @version V1.0
	 * @param User
	 * @return
	 */
	@Select("SELECT PK_T_USER_ID AS ID, USERNAME AS userName, USERPWD AS userPwd, STATUS AS status, "
			+ "LAST_LOGIN_TIME AS lastLoginTime, CREATE_TIME AS createTime, UPDATE_TIME AS updateTime,"
			+ "DEL_FLAG AS delFlag, REALNAME AS realName, LOGINTIMES AS loginTimes FROM T_USER "
			+ "WHERE USERNAME = #{userName} AND USERPWD = #{userPwd} AND (DEL_FLAG = 0 OR DEL_FLAG = 100)")
	User login(User User);

	@SelectKey(statement="select sys_guid() from dual", before = true, keyProperty = "id", resultType = String.class)
	@Insert("INSERT INTO T_USER(PK_T_USER_ID,USERPWD,USERNAME,STATUS,REALNAME,CREATE_TIME,DEL_FLAG,LOGINTIMES,OUTDATE ) "
			+ "VALUES(#{id},#{userPwd},#{userName},#{status},#{realName},#{createTime},#{delFlag},#{loginTimes},#{outDate})")
	int insertUserAndRole(User user);
	
	/**
	 * 连续登录五次锁定
	 * @MethodName:updateLoginTimes
	 * @author: wangyao
	 * @email: 842261834@qq.com 
	 * @date 2017年8月29日 下午1:34:04
	 * @version V1.0
	 * @param user
	 * @return
	 */
	@Update("UPDATE T_USER SET LOGINTIMES = #{loginTimes} WHERE USERNAME = #{userName}")
	int updateLoginTimes(User user); 
	
	/**
	 * 查询该用户名第几次登陆失败
	 * @MethodName:selectDelFlag
	 * @author: wangyao
	 * @email: 842261834@qq.com 
	 * @date 2017年8月29日 下午2:18:06
	 * @version V1.0
	 * @param user
	 * @return
	 */
	@Select("SELECT PK_T_USER_ID AS ID, USERNAME AS userName, USERPWD AS userPwd, STATUS AS status, "
			+ "LAST_LOGIN_TIME AS lastLoginTime, CREATE_TIME AS createTime, UPDATE_TIME AS updateTime,"
			+ "DEL_FLAG AS delFlag, REALNAME AS realName  FROM T_USER WHERE USERNAME = #{userName}")
	User selectDelFlag(String userName);
	
	@Select("SELECT count(*) FROM T_USER_ROLE WHERE ROLE_ID=#{str} AND DEl_FLAG=0")
	int selectRoleToUser(String str); 
	
/*@Select("SELECT PK_T_USER_ID AS ID, USERNAME AS userName, USERPWD AS userPwd, STATUS AS status, "
		+ "LAST_LOGIN_TIME AS lastLoginTime, CREATE_TIME AS createTime, UPDATE_TIME AS updateTime,"
		+ "DEL_FLAG AS delFlag, REALNAME AS realName, LOGINTIMES AS loginTimes  FROM T_USER "
		+ "WHERE USERNAME = #{userName} AND (DEL_FLAG = 0 OR DEL_FLAG = 100)")
User selectLoginTimes(String userName); */
	
	/**
	 * 查询用户登录次数
	 * @MethodName:selectLoginTimes
	 * @author: wangyao
	 * @email: 842261834@qq.com 
	 * @date 2017年9月8日 下午3:03:15
	 * @version V1.0
	 * @param userName
	 * @return
	 */
	@Select("SELECT PK_T_USER_ID AS ID, USERNAME AS userName, USERPWD AS userPwd, STATUS AS status, "
			+ "LAST_LOGIN_TIME AS lastLoginTime, CREATE_TIME AS createTime, UPDATE_TIME AS updateTime,"
			+ "DEL_FLAG AS delFlag, REALNAME AS realName, LOGINTIMES AS loginTimes, OUTDATE AS outDate  FROM T_USER "
			+ "WHERE USERNAME = #{userName} AND (DEL_FLAG = 0 OR DEL_FLAG = 100)")
	User selectLoginTimes(String userName); 
	
	/**
	 * 登陆成功后更改信息
	 * @MethodName:updateLoginInfo
	 * @author: wangyao
	 * @email: 842261834@qq.com 
	 * @date 2017年8月30日 下午5:13:28
	 * @version V1.0
	 * @param user
	 * @return
	 */
	@Update("UPDATE T_USER SET LOGINTIMES = #{loginTimes}, LAST_LOGIN_TIME = #{lastLoginTime}"
			+ "WHERE USERNAME = #{userName}")
	int updateLoginInfo(User user); 
	/**
	 * 用户修改密码
	 * @MethodName:UpdatePwd
	 * @author: wangyao
	 * @email: 842261834@qq.com 
	 * @date 2017年8月31日 上午11:09:08
	 * @version V1.0
	 * @param user
	 * @return
	 */
	@Update("UPDATE T_USER SET USERPWD = #{userPwd}, UPDATE_TIME = #{updateTime},OUTDATE = #{outDate} WHERE PK_T_USER_ID = #{id}")
	int UpdatePwd(User user);

	/**
	 * Author:DuYunbao
	 * 更新所有用户的LOGINTIMES为 0
	 */
	@Update("UPDATE T_USER SET LOGINTIMES = 0 ")
	void updateAllLoginTimes();
	@Update("UPDATE T_USER  SET USERNAME = #{userName},STATUS = #{status},REALNAME = #{realName},LOGINTIMES=#{loginTimes} "
			+ "WHERE  PK_T_USER_ID = #{id}")
	int updateByPrimaryKeySelectiveByUser(User user);
	@Select("SELECT * FROM T_USER tu " +
		    		" INNER JOIN T_USER_ROLE tur on TUR.USER_ID = TU.PK_T_USER_ID"+
		    		" INNER JOIN T_ROLE TR on TR.PK_T_ROLE_ID = TUR.ROLE_ID #{where}")
	List<Map<String, Object>> queryDataWithGridUIAutomatic(String where);
	
	@Select("SELECT COUNT(USERNAME) FROM T_USER WHERE USERNAME =#{userName} AND (DEL_FLAG=#{delFlag} OR DEL_FLAG= 100)")
	int selectByVaUserName(User user);
	
	@Select("SELECT USERNAME FROM T_USER WHERE PK_T_USER_ID =#{userId}")
	String selectUserNew(String userId);
	
	@Select("SELECT REALNAME FROM T_USER WHERE PK_T_USER_ID =#{userId}")
	String selectUserNewTo(String userId);
	
	
	/**
	 * 如果用户被锁用户登录次数清零，用户状态改为停用
	 * @MethodName:updateUserStateWhileLock
	 * @author: wangyao
	 * @email: 842261834@qq.com 
	 * @date 2017年9月8日 下午3:04:59
	 * @version V1.0
	 * @param user
	 * @return
	 */
	@Update("UPDATE T_USER SET STATUS = 0, WHERE USERNAME = #{userName}")
	int updateUserStateWhileLock(User user);
	
	@Select("SELECT * FROM T_USER WHERE PK_T_USER_ID =#{id} AND  USERPWD =#{userPwd}")
	User originPwd(User user);

	@Select("SELECT ROLE_ID FROM T_USER_ROLE WHERE USER_ID =#{id}")
	String selectRoleId(String id);

	@Update("UPDATE T_USER SET STATUS = 0,LOGINTIMES =5 WHERE USERNAME = #{userName}")
	int updateUserStateWhileLockRU(User user);
	
	@Update("UPDATE T_USER SET STATUS = 1,LOGINTIMES =0 WHERE USERNAME = #{userName}")
	int updateUserStateWhileLockRUTo(User user);

	@Select("SELECT * FROM \"T_ROLE\" WHERE DEL_FLAG =0")
	List<Role> queryAllRole();

}	
