package com.catarc.traceability.util;

import com.catarc.traceability.entity.ComponentsCode;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by Administrator on 2017/8/25.
 */
public class ReadExcel {


    //总行数
    private int totalRows = 0;
    //总条数
    private int totalCells = 0;
    //错误信息接收器
    private String errorMsg;

    //构造方法
    public ReadExcel() {
    }

    //获取总行数
    public int getTotalRows() {
        return totalRows;
    }

    //获取总列数
    public int getTotalCells() {
        return totalCells;
    }

    //获取错误信息
    public String getErrorInfo() {
        return errorMsg;
    }

    /**
     * 验证EXCEL文件
     *
     * @param filePath
     * @return
     */
    public boolean validateExcel(String filePath) {
        if (filePath == null || !(WDWUtil.isExcel2003(filePath) || WDWUtil.isExcel2007(filePath))) {
            errorMsg = "文件名不是excel格式";
            return false;
        }
        return true;
    }

    /**
     * 读EXCEL文件，获取客户信息集合
     *
     * @param fileName
     * @param Mfile
     * @return
     */
    private Workbook getExcelInfo(String fileName, MultipartFile Mfile) {
        Workbook wb = null;
        //把spring文件上传的MultipartFile转换成CommonsMultipartFile类型
        CommonsMultipartFile cf = (CommonsMultipartFile) Mfile; //获取本地存储路径
        //初始化输入流
        InputStream is = null;
        try {
            //根据文件名判断文件是2003版本还是2007版本
            boolean isExcel2003 = true;
            if (WDWUtil.isExcel2007(fileName)) {
                isExcel2003 = false;
            }
            is = cf.getInputStream();
            //根据excel里面的内容读取客户信息
            wb = getExcelInfo(is, isExcel2003, wb);
            is.close();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException e) {
                    is = null;
                    e.printStackTrace();
                }
            }
        }
        return wb;
    }

    /**
     * 判断excel版本
     *
     * @param is
     * @param isExcel2003
     * @param wb
     * @return
     */
    private Workbook getExcelInfo(InputStream is, boolean isExcel2003, Workbook wb) {
        //List<ComponentsCode> ComponentsCodeList = null;
        try {
            /** 根据版本选择创建Workbook的方式 */
            //当excel是2003时
            if (isExcel2003) {
                wb = new HSSFWorkbook(is);
            } else {//当excel是2007时
                wb = new XSSFWorkbook(is);
            }
            //读取Excel里面客户的信息
            // readExcelValue(wb);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return wb;
    }

    /**
     * 读取Excel里面的信息
     *
     * @param fileName
     * @param Mfile
     * @return
     */
    public List<ComponentsCode> readExcelValue(String fileName, MultipartFile Mfile) {

        Workbook wb = getExcelInfo(fileName, Mfile);

        List<ComponentsCode> componentscodeList = new ArrayList<ComponentsCode>();
        //得到每一个shell
        for (int i = 0; i < wb.getNumberOfSheets(); i++) {
            Sheet sheet = wb.getSheetAt(i);

            //得到Excel的行数
            this.totalRows = sheet.getPhysicalNumberOfRows();

            //得到Excel的列数(前提是有行数)
            if (totalRows >= 1 && sheet.getRow(0) != null) {
                this.totalCells = sheet.getRow(0).getPhysicalNumberOfCells();
            }

            //循环Excel行数,从第二行开始。标题不入库
            for (int r = 1; r < totalRows; r++) {
                Row row = sheet.getRow(r);
                if (row == null) continue;
                ComponentsCode componentscode = new ComponentsCode();
                //循环Excel的列
                for (int c = 0; c < this.totalCells; c++) {
                    Cell cell = row.getCell(c);
                    if (cell != null) {
                        cell.setCellType(Cell.CELL_TYPE_STRING);
                        if (c == 0) {//第一列
                            componentscode.setCodeId(cell.getStringCellValue());//零件识别码
                        }
                    }
                }
                //添加客户
                if (StringUtils.isNotEmpty(componentscode.getCodeId())) { //当传入的excel并不符合我们的要求时，解析后的数据会是空
                    componentscodeList.add(componentscode);
                }
            }
        }
        return componentscodeList;
    }
}
